# Copyright with the authors of the publication "A principal feature analysis"

import networkx as nx
import numpy as np
import scipy.stats
import random
from multiprocessing import Pool


# see paper Algorithm 2
def principal_feature_analysis(cluster_size,data,number_output_functions,freq_data,l,left_features,alpha,shuffle_feature_numbers):
    number_nodes= len(left_features) - number_output_functions  # Subtract the number of components of the output function
    list_of_nodes= left_features[number_output_functions:].copy()  # Take only the features and not the components of the output function
    m= data.shape[0]    # number of rows of the data matrix
    n = data.shape[1]   # number of columns of the data matrix
    number_chisquare_tests=0    # number of total chi-square tests
    counter_bin_less_than5 = 0  # number of chi-square tests with a bin less than 5 data points
    counter_bin_less_than1 = 0  # number of chi-square tests with a bin less than 1 data point
    global_adjm=np.zeros((m,m)) # global adjacency matrix
    is_entry_calculated=np.zeros((m,m)) # flag if the corresponding entry of the global adjacency matrix is already calculated

    with Pool() as pool:
        while(True):
            print("Nodes left: " + str(number_nodes))
            list_of_clusters=[]
            intermediate_list=[]
            if shuffle_feature_numbers==1:
                random.shuffle(list_of_nodes)  # Pick the nodes of the subgraphs randomly if uncommented
            # Cluster the nodes into subsets of nodes with at most cluster_size nodes
            for i in range(0,int(number_nodes/cluster_size)+1):
                for j in range(0,cluster_size):
                    if (i * cluster_size + j) < number_nodes:
                        intermediate_list.append(list_of_nodes[i * cluster_size + j])
                if intermediate_list != []:
                    list_of_clusters.append(sorted(intermediate_list))
                    intermediate_list=[]
            list_of_nodes=[]

            any_cluster_dissected=0  #  flag to notify if no further node was removed from any subgraph
            
            # for all clusters...
            for cluster in list_of_clusters:
                number_elements_cluster = len(cluster)
                counter_calculations = 0

                # ...calculate adjacency matrix using chi-square test...
                for i in range(0, number_elements_cluster):
                    for j in range(i + 1, number_elements_cluster):
                        # if a chi-square test for the corresponding pair of nodes is already performed, the result from the last calculation is used, else the chi-square test is performed
                        # if both tested random features have more than one output value in their discretization because in this case the features are always independent
                        if is_entry_calculated[cluster[i],cluster[j]]==0 and (len(freq_data[cluster[i]])>1 and len(freq_data[cluster[j]])>1):
                            is_entry_calculated[cluster[i],cluster[j]]=1
                            is_entry_calculated[cluster[j],cluster[i]]=1
                            number_chisquare_tests +=1
                            freq_data_product = np.histogram2d(data[cluster[i], :], data[cluster[j], :],
                                                            bins=(l[cluster[i]], l[cluster[j]]))[0]
                            expfreq = np.outer(freq_data[cluster[i]], freq_data[cluster[j]]) / n

                            if sum(expfreq.flatten() < 5) > 0:
                                counter_bin_less_than5 += 1
                            if sum(expfreq.flatten() < 1) > 0:
                                counter_bin_less_than1 += 1
                            pv = scipy.stats.chisquare(freq_data_product.flatten(), expfreq.flatten(),ddof=-1)[1]
                            # ddof=-1 to have the degrees of freedom of the chi square eaual the number of bins, see corresponding paper (Appendix) for details
                            # if p-value pv is less than alpha the hypothesis that j is independent of i is rejected
                            if pv <= alpha:
                                global_adjm[cluster[i], cluster[j] ] = 1
                                global_adjm[cluster[j], cluster[i]] = 1
                        counter_calculations += 1
                adjm=(global_adjm[cluster,:])[:,cluster] # add the entries of the adjacency matrix of the subcluster to the global adjacency matrix
                
                # see paper Algorithm 1
                # ...create subgraphs based on adjacency matrix
                G=nx.from_numpy_matrix(adjm)
                S = [G.subgraph(c).copy() for c in nx.connected_components(G)]

                list_graphs_to_divide=[]    # list of graphs to divide
                list_complete_sub_graphs=[] # list of complete subgraphs
                list_nodes_complete_sub_graphs=[]   # list of lists of nodes corresponding to the complete subgraphs of list_complete_sub_graphs
            
        
                # filter non-complete subgraphs
                for i in S:
                    if list(nx.complement(i).edges)!=[]: # if a graph is not complete
                        list_graphs_to_divide.append(i)
                    else:
                        list_complete_sub_graphs.append(i)
                        list_nodes_complete_sub_graphs.append(list(i.nodes))


                idx = 0
    
                set_nodes_to_delete = pool.map(nx.minimum_node_cut, list_graphs_to_divide)
                assert len(set_nodes_to_delete) == len(list_graphs_to_divide)
        
                global_list_graphs_to_divide = list_graphs_to_divide.copy()
                window = len(set_nodes_to_delete)
                
                if list_graphs_to_divide!=[]:
                    any_cluster_dissected=1
                    
                for current_graph in list_graphs_to_divide:
                    print(str(len(set_nodes_to_delete[idx])) + " nodes removed!")
                    
                    for node in list(set_nodes_to_delete[idx]):
                        current_graph.remove_node(node)                     # remove the nodes that were found with the minimum cut algorithm
                    list_new_sub_graphs = [current_graph.subgraph(c).copy() for c in nx.connected_components(current_graph)]
                    # Sort the new subgraphs into a list of complete subgraphs and subgraphs that can be further divided
                    for sub_graph_of_current_graph in list_new_sub_graphs:
                        if list(nx.complement(sub_graph_of_current_graph).edges)!=[]:
                            list_graphs_to_divide.append(sub_graph_of_current_graph)
                            global_list_graphs_to_divide.append(sub_graph_of_current_graph)
                        else:
                            list_complete_sub_graphs.append(sub_graph_of_current_graph)
                            list_nodes_complete_sub_graphs.append(list(sub_graph_of_current_graph.nodes))
                    idx += 1
                    
                    if idx >= window:
                        set_nodes_to_delete += pool.map(nx.minimum_node_cut, global_list_graphs_to_divide[idx:])
                        window = len(set_nodes_to_delete)
                            
                
                
                # Transform the numbering of the nodes of the subcluster back to the global numbering
                for sub_list in list_nodes_complete_sub_graphs:
                    for node_of_sub_list in sub_list:
                        list_of_nodes.append(cluster[node_of_sub_list])
            list_of_nodes.sort()
            number_nodes=len(list_of_nodes)     # Update the number of left features
            if any_cluster_dissected==0:    # If no subgraph was dissected set cluster_size = number_nodes to consider the total graph of the remaining features at once
                cluster_size=number_nodes
            if len(list_of_clusters)<=1:    # If the total graph of all features has been considered stop
                break
    print("Nodes left: " + str(number_nodes))
    print('Dissection done!')
    return list_nodes_complete_sub_graphs, counter_bin_less_than5 / number_chisquare_tests * 100, counter_bin_less_than1 / number_chisquare_tests * 100



# Copyright with the authors of the publication "A principal feature analysis"


from .find_relevant_principal_features import find_relevant_principal_features
from .get_mutual_information import get_mutual_information
import time
import pandas as pd
import numpy as np


# paramters for the PFA
# number_output_functions: Number of output features that are to be modeled, i.e. the number of components of the vector-valued output-function. The values are stored in the first number_output_functions rows of the csv-file.
# number_sweeps: Number of sweeps of the PFA. The result of the last sweep is returned.
                # In addition, the return of each sweep are interesected and returned as well.
# cluster_size: number of nodes of a subgraph in the principal_feature_analysis
# alpha=0.01: Level of significance
# min_n_datapoints_a_bin: minimum number of data points for each bin in the chi-square test
# shuffle_feature_numbers: if True the number of the features is randomly shuffled
# frac: the fraction of the dataset that is used for the analysis. The set is randomly sampled from the input csv
# calculate_mutual_information: if True the mutual information with features from the PFA with the components of the output function is calculated
# basis_log_mutual_information: basis of the logarithm used in the calculation of the mutual information




def par_pfa(path, number_output_functions=1, number_sweeps=1, cluster_size=50, alpha=0.01, min_n_datapoints_a_bin=500, shuffle_feature_numbers=0, frac=1, calculate_mutual_information=0, basis_log_mutual_information=2):
    # pf_ds = principal features related to output functions, pf = all principal features
    start_time=time.time()
    list_pf_ds=[]


    # The csv file's content is an m x n Matrix with m - number components of output-function = number features and n = number of data points
    # where the first number components of output-function rows contain the value of the vector-valued output function for each of the n data points
    # e.g. in case of a one-dimensional output function, the first row can be the label for each data point
    data = pd.read_csv(path, sep=',', header=None)
    for sweep in range(0,number_sweeps):
        print("Sweep number: " + str(sweep+1))
        pf_ds,pf,indices_principal_feature_values=find_relevant_principal_features(data,number_output_functions,cluster_size,alpha,min_n_datapoints_a_bin,shuffle_feature_numbers,frac)
        list_pf_ds.append(pf_ds)
        # Output the principal features related to the output function in a list where the numbers correspod to the rows of the input csv-file
        f = open("principal_features_depending_system_state"+str(sweep)+".txt", "w")
        for i in pf_ds:
            for j in i:
                f.write(str(j) + str(","))
            f.write("\n")
        f.close()
        # Output the principal features in a list where the numbers correspond to the rows of the input csv-file
        f = open("principal_features_global_indices"+str(sweep)+".txt", "w")
        for i in pf:
            for j in i:
                f.write(str(j) + str(","))
            f.write("\n")
        f.close()
        np.savetxt("global_indices_and_principal_features_state_dependency"+str(sweep)+".csv", indices_principal_feature_values,delimiter=",")
    print("Time needed for the PFA in seconds: " + str(time.time()-start_time))


    #Intersect the lists of principal features related to the output function
    #All the features corresponding to the returned subgraphs are considered in each list

    list_principal_features_depending_on_system_state_for_intersection=[]
    for i in list_pf_ds:
        intermediate_list = []
        for j in i:
            for k in j:
                if k !='*':
                    intermediate_list.append(k)
        list_principal_features_depending_on_system_state_for_intersection.append(intermediate_list)
    pf_from_intersection=list_principal_features_depending_on_system_state_for_intersection[0]
    if number_sweeps > 1:
        for i in range(1, len(list_principal_features_depending_on_system_state_for_intersection)):
            pf_from_intersection=list(set(pf_from_intersection).intersection(set(list_principal_features_depending_on_system_state_for_intersection[i])))
        f = open("principal_features_depending_system_state_intersection.txt", "w")
        for i in pf_from_intersection:
            f.write(str(i)+str(","))
        f.close()

    # Outputs a list of DataFrames where the index feature refers to the row in the csv-file.
    # The mutual information is calculated between the feature represented in the first row of the data frame and the feature referred to in the index feature column.
    # The first row is consequently the mutual information of the corresponding component of the output-function with itself
    if calculate_mutual_information==1:
        print("Calculating mutual information")
        list_data_frame_feature_mutual_information=get_mutual_information(data,number_output_functions,pf_from_intersection,min_n_datapoints_a_bin,basis_log_mutual_information)
        for i in range(0,len(list_data_frame_feature_mutual_information)):
            print(list_data_frame_feature_mutual_information[i])
        return pf_from_intersection, list_data_frame_feature_mutual_information
        
    return pf_from_intersection



# Copyright with the authors of the publication "A principal feature analysis"

import numpy as np
import scipy.stats
import pandas as pd
from .principal_feature_analysis import principal_feature_analysis

def find_relevant_principal_features(data,number_output_functions,cluster_size,alpha,min_n_datapoints_a_bin,shuffle_feature_numbers,frac):
    # In this function the binning is done, the graph is dissected and the relevant features/variables are detected
    if frac<1: # if frac<1 the feature analysis is done only the fraction of the data, randomly sampled
        data=data.sample(frac=frac,axis='columns',replace=False)
    data=data.to_numpy()
    m = data.shape[0] #number features
    n = data.shape[1] #number of data points
    l = [0 for i in range(0, m)]  # list of lists with the points of support for the binning
    freq_data = [0 for i in range(0, m)] # list of histograms
    left_features = [i for i in range(0, m)]  # list of features that is step by step reduced to the relevant ones
    constant_features = []

    # remove constant features and binning (discretizing the continuous values of our features)
    for i in range(0, m): 
        mindata = min(data[i, :])
        maxdata = max(data[i, :])
        if maxdata <= mindata:
            print("Feature #"f"{i}" " has only constant values")
            left_features.remove(i)
            constant_features.append(i)
        else:
            # start the binning by sorting the data points
            list_points_of_support = []
            datapoints = data[i, :].copy()
            datapoints.sort()
            last_index = 0
            # go through the data points and bin them
            for point in range(0, datapoints.size):
                if point >= (datapoints.size - 1):  # if end of the data points leave the for-loop
                    break
                # close a bin if there are at least min_n_datapoints_a_bin and the next value is bigger
                if datapoints[last_index:point + 1].size >= min_n_datapoints_a_bin and datapoints[point] < datapoints[point + 1]:
                    list_points_of_support.append(datapoints[point + 1])
                    last_index = point + 1
            if len(list_points_of_support) > 0: # test that there is at least one point of support (it can be if there are only constant value up to the first ones which are less than min_n_datapoints_a_bin
                if list_points_of_support[0] > datapoints[0]: # add the first value as a point of support if it does not exist (less than min_n_datapoints_a_bin at the beginning)
                    list_points_of_support.insert(0, datapoints[0])
            else:
                list_points_of_support.append(datapoints[0])
            list_points_of_support.append(datapoints[-1] + 0.1) # Add last point of support such that last data point is included (half open interals in Python!)
            if datapoints[datapoints >= list_points_of_support[-2]].size < min_n_datapoints_a_bin: # if last bin has not at least min_n_datapoints_a_bin fuse it with the one before the last bin
                if len(list_points_of_support) > 2:     # Test if there are at least 3 points of support (only two can happen if there only constant values at the beginning and only less than n_min_datapoints_a_bin in the end)
                    list_points_of_support.pop(-2)
            l[i] = list_points_of_support
            freq_data[i] = np.histogram(data[i, :], bins=l[i])[0]
    print("Binning done!")
    print("List of features with constant values:")
    print(constant_features)
    for id_output in range(0,number_output_functions):
        if id_output in constant_features or len(freq_data[id_output]) < 2:  # Warn if the output function is constant e.g. due to an unsuitable binning
            print("Warning: System state " + str(id_output) +  " is constant!")

    print("Starting principal feature analysis!")
    # list_principal_features = list of resulting principal features
    # smaller_than5 = percent of chi-square tests with bins less than 5 data points
    # smaller_than1 = percent of chi-square tests with bins less than 1 data point
    list_principal_features, smaller_than5, smaller_than1 = principal_feature_analysis(cluster_size, data, number_output_functions, freq_data, l, left_features, alpha, shuffle_feature_numbers)

    # Assign global index to each principal feature
    list_principal_features_global_indices = []
    for i in list_principal_features:
        intermediate_list = []
        for j in i:
            shift_variable = j + number_output_functions  # Output functions, which are not considered in the PFA, are the values 0,1,...,number_output_functions-1 in left_features
            intermediate_list.append(left_features[shift_variable])
        list_principal_features_global_indices.append(intermediate_list)

    print('principal features:')
    print(list_principal_features_global_indices)



    # identify principal features related to the output function again using chi-square
    print("Start calculating dependence on system state")
    principal_features_depending_on_system_state=[]
    principal_features_not_depending_on_system_state = []
    counter_bins_less_than5_relevant_principal_features=0 # number of chi-square tests with less than 5 datapoints a bin
    counter_bins_less_than1_relevant_principal_features=0 # number of chi-square tests with less than 1 datapoint a bin
    counter_number_chi_square_tests_relevant_principal_features=0 # number of total chi-square tests
    indices_principal_feature_values=np.zeros((1, 2))

    for i in list_principal_features_global_indices: # go through any complete graph
        intermediate_list_depending_on_system_state=[]
        intermediate_list_not_depending_on_system_state=[]
        # for each feature within the current complete graph perform a chi-square test
        for j in i:
            if len(freq_data[j]) > 1:
                dependent = 0 # Flag for the input feature j if there is a relation to one output-function
                for id_output in range(0,number_output_functions):
                    counter_number_chi_square_tests_relevant_principal_features +=1
                    freq_data_product = np.histogram2d(data[id_output, :], data[j, :],
                                                bins=(l[id_output], l[j]))[0]
                    expfreq = np.outer(freq_data[id_output], freq_data[j]) / n
                    if sum(expfreq.flatten() < 5) > 0:
                        counter_bins_less_than5_relevant_principal_features += 1
                    if sum(expfreq.flatten() < 1) > 0:
                        counter_bins_less_than1_relevant_principal_features += 1
                    pv = scipy.stats.chisquare(freq_data_product.flatten(), expfreq.flatten(),ddof=-1)[1]
                    # ddof=-1 to have the degrees of freedom of the chi square eaual the number of bins, see corresponding paper (Appendix) for details
                    # if p-value pv is less than alpha the hypothesis that j is independent of the output function is rejected
                    if pv <= alpha:
                        dependent=1
                        break
                if dependent==1:
                    intermediate_list_depending_on_system_state.append(j)
                else:
                    intermediate_list_not_depending_on_system_state.append(j)
            else:
                intermediate_list_not_depending_on_system_state.append(j)
                pv=1.1
            indices_principal_feature_values=np.concatenate((indices_principal_feature_values, np.array([j, pv]).reshape((1, 2))), axis=0)

        #Output the result
        intermediate_list_depending_on_system_state_empty = 1
        intermediate_list_not_depending_on_system_state_empty=1
        if len(intermediate_list_depending_on_system_state)>0:
            intermediate_list_depending_on_system_state_empty=0
        if len(intermediate_list_not_depending_on_system_state)>0:
            intermediate_list_not_depending_on_system_state_empty=0
            principal_features_not_depending_on_system_state.append(intermediate_list_not_depending_on_system_state)

        if intermediate_list_depending_on_system_state_empty==0:
            if intermediate_list_depending_on_system_state_empty==intermediate_list_not_depending_on_system_state_empty:
                print("Ambiguous complete subgraph:")
                print("List with features depending on system state:")
                print(intermediate_list_depending_on_system_state)
                print("List with features not depending on system state:")
                print(intermediate_list_not_depending_on_system_state)
                # if a subgraph contains a node not independent of the output function and a node independent of the node, the classes of nodes are separated by *
                intermediate_list_depending_on_system_state = intermediate_list_depending_on_system_state + ['*'] + intermediate_list_not_depending_on_system_state
            principal_features_depending_on_system_state.append(intermediate_list_depending_on_system_state)
    print("principal features depending on system state:")
    print(principal_features_depending_on_system_state)
    print(str(smaller_than5) + "% of the chi-square test for the adjacency matrix have been performed with a bin less than 5 data points!")
    print(str(smaller_than1) + "% of the chi-square test for the adjacency matrix have been performed with a bin less than 1 data points!")
    print(str(counter_bins_less_than5_relevant_principal_features / counter_number_chi_square_tests_relevant_principal_features * 100) + "% of the chi-square test for finding the relevant principal features have been performed with a bin less than 5 data points!")
    print(str(counter_bins_less_than1_relevant_principal_features / counter_number_chi_square_tests_relevant_principal_features * 100) + "% of the chi-square test for finding the relevant principal features have been performed with a bin less than 1 data points!")
    return principal_features_depending_on_system_state, list_principal_features_global_indices, indices_principal_feature_values



# Copyright with the authors of the publication "A principal feature analysis"

import numpy as np
import pandas as pd
import math

# Function to calculate the Shannon mutal information of features with the components of the output function
# Input:    data: a csv file in which the first number of components of the output function rows are the values of the components of the output function  and the other rows are the values of the features
#           list_variables: sorted list of indices that refer to the features stored in the row of the csv-file
#           min_n_datapoints_a_bin: is supposed to have the same value as min_n_datapoints_a_bin in the function find_relevant_principle_features
#           basis_log_mutual_information:  the basis for the logarithm used to calculate the mutual information.
# Output:   list_pd_mutual_information: list of pandas data frames where each the first column refers to the indices of the row in the csv (respectively, the variable of which the values are stored there) and the second column refers to the corresponding mutual information of the corresponding feature with the component of the output-function (indicated in the first row of first column)
#                                  The first entry is the mutual information of the corresponding component of the output fuction with itself. The further entries are the mutual information of the corresponding row in the csv-file to the corresponding component of the output function

def get_mutual_information(data,number_output_functions,list_variables,min_n_datapoints_a_bin,basis_log_mutual_information):
    # Calulate the Shannon mutual information
    def make_summand_from_frequencies(x,y):
        if x == 0:
            return 0
        else:
            return x * math.log2( x / y) / math.log2(basis_log_mutual_information)

    # Insert the the indices of the rows where the components of the output functions are stored
    for i in range(0,number_output_functions):
        list_variables.insert(i,i)

    data_init = data.to_numpy()
    data=data_init[list_variables,:]
    m = data.shape[0]
    n = data.shape[1]
    l = [0 for i in range(0, m)]
    freq_data = [0 for i in range(0, m)]
    left_features = [i for i in range(0, m)]
    constant_features = []

    for i in range(0, m): 
        mindata = min(data[i, :])
        maxdata = max(data[i, :])
        if maxdata <= mindata:
            print("Feature #"f"{list_variables[i]}" " has only constant values")
            left_features.remove(i)
            constant_features.append(list_variables[i])
        else:
            # start the binning by sorting the data points
            list_points_of_support = []
            datapoints = data[i, :].copy()
            datapoints.sort()
            last_index = 0
            # go through the data points and bin them
            for point in range(0, datapoints.size):
                if point >= (datapoints.size - 1):  # if end of the data points leave the for-loop
                    break
                # close a bin if there are at least min_n_datapoints_a_bin and the next value is bigger
                if datapoints[last_index:point + 1].size >= min_n_datapoints_a_bin and datapoints[point] < datapoints[point + 1]:
                    list_points_of_support.append(datapoints[point + 1])
                    last_index = point + 1
            if len(list_points_of_support) > 0: # test that there is at least one point of support (it can be if there are only constant value up to the first ones which are less than min_n_datapoints_a_bin
                if list_points_of_support[0] > datapoints[0]: # add the first value as a point of support if it does not exist (less than min_n_datapoints_a_bin at the beginning)
                    list_points_of_support.insert(0, datapoints[0])
            else:
                list_points_of_support.append(datapoints[0])
            list_points_of_support.append(datapoints[-1] + 0.1) # Add last point of support such that last data point is included (half open interals in Python!)
            if datapoints[datapoints >= list_points_of_support[-2]].size < min_n_datapoints_a_bin: # if last bin has not at least min_n_datapoints_a_bin fuse it with the one before the last bin
                if len(list_points_of_support) > 2:     # Test if there are at least 3 points of support (only two can happen if there only constant values at the beginning and only less than n_min_datapoints_a_bin in the end)
                    list_points_of_support.pop(-2)
            l[i] = list_points_of_support
            freq_data[i] = np.histogram(data[i, :], bins=l[i])[0]


    #Check for constant features
    if constant_features != []:
        print("List of features with constant values:")
        print(constant_features)
    for id_output in range(0,number_output_functions):
        if id_output in constant_features or len(freq_data[id_output]) < 2:  # Warn if the output function is constant e.g. due to an unsuitable binning
            print("Warning: Output function " + str(id_output) +  " is constant!")

    # Calculate the mutual information for each feature with the corresponding component of the output function
    list_of_data_frames=[]
    mutual_info = np.ones((1, len(left_features) - number_output_functions + 1))  # number of featuers plus one component of the output-function
    for i in range(0,number_output_functions):
        list_of_features=list(range(number_output_functions,len(left_features)))
        list_of_features.insert(0,i)
        id_features=np.array(list_variables)[left_features]
        
        for j in list_of_features:
            freq_data_product = ((np.histogram2d(data[i,:], data[left_features[j], :], bins=(l[i], l[left_features[j]]))[0])) / n
            expfreq = (np.outer(freq_data[i], freq_data[left_features[j]])) / (n * n)
            if j<number_output_functions:
                mutual_info[0, 0] = np.sum(np.array(list(map(make_summand_from_frequencies, freq_data_product.flatten().tolist(),expfreq.flatten().tolist()))))
            else:
                mutual_info[0,j-number_output_functions+1]=np.sum(np.array(list(map(make_summand_from_frequencies,freq_data_product.flatten().tolist(),expfreq.flatten().tolist()))))
        pd_mutual_information = pd.DataFrame({"index feature": id_features, "mutual information": mutual_info.tolist()[0]})
        pd_mutual_information['index feature'] = pd_mutual_information['index feature'].astype(int)
        list_of_data_frames.append(pd_mutual_information)

    return list_of_data_frames



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="parallel-principal-feature-analysis",
    version="1.0.2",
    author="Tim Breitenbach & Lauritz Rasbach",
    author_email="tim.breitenbach@mathematik.uni-wuerzburg.de, rasbachlauritz@googlemail.com",
    description="The first package for (parallel) Principal Feature Analysis",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
    install_requires=['numpy','scipy','pandas','networkx']
)



from .execute_PFA import par_pfa
from .find_relevant_principal_features import find_relevant_principal_features
from .get_mutual_information import get_mutual_information
from .principal_feature_analysis import principal_feature_analysis

